/*
 *								=== Utils.h ===
 *
 *		- note this is supposed to be included once per app, to main.cpp
 */

#ifndef __UTILS_INCLUDED
#define __UTILS_INCLUDED

#if defined(_MSC_VER) && !defined(__MWERKS__) && !defined(for)
#define for if(0) {} else for
#endif

#include "../UberLame_src/NewFix.h"
#include "../UberLame_src/CallStack.h"
#include "../UberLame_src/Thread.h"
#include "../UberLame_src/Socket.h"
#include "../UberLame_src/proto/HTTP_Client.h"

class CFileReceiver {
protected:
	const char *m_p_s_output;
	FILE *m_p_fw;

public:
	CFileReceiver(const char *p_s_output)
		:m_p_s_output(p_s_output), m_p_fw(0)
	{}

	~CFileReceiver()
	{
		if(m_p_fw) {
			fclose(m_p_fw);
			m_p_fw = 0;
		}
	}

	bool operator ()(const void *p_data, unsigned int n_size)
	{
		if(!m_p_fw) {
			if(!(m_p_fw = fopen(m_p_s_output, "wb")))
				return false;
		}
		return fwrite(p_data, n_size, 1, m_p_fw) == 1;
	}
};

enum {
	down_Waiting = 0,
	down_Failure,
	down_FileDoesntExist,
	down_Success
};

class CDownloader : public CRunable {
protected:
	std::string m_s_filename;
	std::string m_s_url;
	int m_n_status;
	bool m_b_assignment;
	CThread m_thread;
	const char *m_p_s_proxy;

public:
	CDownloader()
		:m_n_status(down_Waiting), m_b_assignment(false), m_p_s_proxy(0)
	{
		m_thread.AttachRunable(*this);
	}

	~CDownloader()
	{
		m_thread.Stop(true);
		// kill the thread
	}

	bool StartDownload(const char *p_s_filename, const std::string &r_s_url)
	{
		if(m_b_assignment || m_thread.b_IsRunning())
			return false;
		// busy

		m_s_filename.reserve(strlen(p_s_filename));
		if(m_s_filename.capacity() < strlen(p_s_filename)) {
			fprintf(stderr, "error: not enough memory 0\n");
			return false;
		}
		m_s_filename = p_s_filename;
		m_s_url.reserve(r_s_url.length() + 256);
		if(m_s_url.capacity() < r_s_url.length()) {
			fprintf(stderr, "error: not enough memory 1\n");
			return false;
		}
		m_s_url = r_s_url;
		// copy the strings

		if(!m_thread.Start()) {
			fprintf(stderr, "error: failed to start a new thread\n");
			return false;
		}
		// start the thread

		m_b_assignment = true;

		return true;
	}

	void SetProxy(const char *p_s_proxy)
	{
		m_p_s_proxy = p_s_proxy;
	}

	int n_WaitForFinish()
	{
		if(!m_b_assignment)
			return false; // was not downloading anything
		if(!m_thread.Stop()) {
			m_b_assignment = false; // fixme?
			return false;
		}
		m_b_assignment = false;
		return m_n_status;
	}

	bool b_Free() const
	{
		return !m_b_assignment;
	}

	bool b_Busy()
	{
		return m_b_assignment && m_thread.b_IsRunning();
	}

	bool b_Finished()
	{
		return m_b_assignment && !m_thread.b_IsRunning();
	}

	const std::string &r_s_FileName() const
	{
		return m_s_filename;
	}

	const std::string &r_s_Url() const
	{
		return m_s_url;
	}

	virtual void Run()
	{
		m_n_status = n_Download_FILE(m_s_filename.c_str(), m_s_url);
	}

protected:
	static bool TrashData(const void *p_data, unsigned int n_size)
	{
		fprintf(stderr, "warning: downloading %d excess bytes\n", n_size);
		return true;
	}

	int n_Download_FILE(const char *p_s_output, std::string &r_s_url)
	{
		CHTTP_File my_file(r_s_url.c_str());
		if(!my_file.p_s_Location()) {
			fprintf(stderr, "error: not enough memory 2\n");
			return down_Failure;
		}
		// prepare

		bool b_result;
		if(m_p_s_proxy) {
			for(int n_retry = 0; n_retry < 5; ++ n_retry) {
				if((b_result = my_file.ProxiedReceive(m_p_s_proxy, CFileReceiver(p_s_output))))
					break;
			}
		} else
			b_result = my_file.Receive(CFileReceiver(p_s_output));
		if(!my_file.p_s_Location()) {
			//fprintf(stderr, "error: internet connection unavailable\n");
			return down_Failure;
		}
		// let itself redirect

		if(!my_file.p_Header() || my_file.p_Header()->n_Status_Code() < 200 ||
		   my_file.p_Header()->n_Status_Code() >= 400)
			return down_FileDoesntExist; // down_Failure
		// it should end with http 300, 200 would be ok too

		/*char p_s_test[64];
		sprintf(p_s_test, "/film/%d-", n_index);
		if(!strstr(my_file.p_s_Location(), p_s_test))
			return down_FileDoesntExist;
		// we were redirected away*/

		if(!b_result) {
			/*if(my_file.p_Header()) {
				fprintf(stderr, "error: failed to receive \'%s\' "
					"(HTTP %03d)\n", r_s_url.c_str(), my_file.p_Header()->n_Status_Code());
			} else {
				fprintf(stderr, "error: failed to receive \'%s\' "
					"(are you connected to the internet?)\n", r_s_url.c_str());
			}*/
			return down_Failure;
		}
		// downloaded already
		// receive

		r_s_url.reserve(strlen(my_file.p_s_Location()));
		if(r_s_url.capacity() < strlen(my_file.p_s_Location())) {
			fprintf(stderr, "error: not enough memory 3 '%s'\n", my_file.p_s_Location());
			return down_Failure;
		}
		r_s_url.assign(my_file.p_s_Location());
		// remember url (must not realloc; do it later in case CFileReceiver failed)

		return down_Success;
	}
};

/*class CWatchDog : public CRunable {
protected:
	static int m_n_counter;
	int m_n_counter_sample;
	int m_n_check_period;
	CThread m_thread;

public:
	CWatchDog(int n_check_period_milliseconds)
		:m_n_counter_sample(m_n_counter),
		m_n_check_period(n_check_period_milliseconds)
	{
		m_thread.AttachRunable(*this);
	}

	bool Enable()
	{
		if(m_thread.b_IsRunning())
			return true;
		return m_thread.Start();
	}

	bool Disable()
	{
		if(!m_thread.b_IsRunning())
			return true;
		return m_thread.Stop(true);
	}

	static void NotifyRunning()
	{
		++ m_n_counter;
	}

	virtual void Run()
	{
		for(;;) {
			m_n_counter_sample = m_n_counter;
			Sleep(m_n_check_period);
			if(m_n_counter_sample == m_n_counter) {
				fprintf(stderr, "error: program halted\n");
				exit(-1);
			}
		}
	}
};

int CWatchDog::m_n_counter = 0;*/

#endif //__UTILS_INCLUDED
