//                              --- OpenGL 1.4 ---

/**
 *  @def GL_VERSION_1_4
 *  @brief if defined, OpenGL 1.4 entry points are available
 *  @note This does not necessarily mean it is also supported.
 */
#ifndef GL_VERSION_1_4
#define GL_VERSION_1_4 1

/**
 *  @def __GENERATE_GL_14__
 *  @brief if defined, code to initialize OpenGL 1.4 entry points is compiled
 */
#define __GENERATE_GL_14__

extern bool GLEH_OPENGL_1_4; /**< @brief set if OpenGL 1.4 is supported */

#define GL_BLEND_DST_RGB                                            0x80C8 /**< @brief OpenGL 1.4 enum */
#define GL_BLEND_SRC_RGB                                            0x80C9 /**< @brief OpenGL 1.4 enum */
#define GL_BLEND_DST_ALPHA                                          0x80CA /**< @brief OpenGL 1.4 enum */
#define GL_BLEND_SRC_ALPHA                                          0x80CB /**< @brief OpenGL 1.4 enum */
//#define GL_POINT_SIZE_MIN                                           0x8126 // deprecated
//#define GL_POINT_SIZE_MAX                                           0x8127 // deprecated
#define GL_POINT_FADE_THRESHOLD_SIZE                                0x8128 /**< @brief OpenGL 1.4 enum */
//#define GL_POINT_DISTANCE_ATTENUATION                               0x8129 // deprecated
//#define GL_GENERATE_MIPMAP                                          0x8191 // deprecated
//#define GL_GENERATE_MIPMAP_HINT                                     0x8192 // deprecated
#define GL_DEPTH_COMPONENT16                                        0x81A5 /**< @brief OpenGL 1.4 enum */
#define GL_DEPTH_COMPONENT24                                        0x81A6 /**< @brief OpenGL 1.4 enum */
#define GL_DEPTH_COMPONENT32                                        0x81A7 /**< @brief OpenGL 1.4 enum */
#define GL_MIRRORED_REPEAT                                          0x8370 /**< @brief OpenGL 1.4 enum */
//#define GL_FOG_COORDINATE_SOURCE                                    0x8450 // deprecated
//#define GL_FOG_COORDINATE                                           0x8451 // deprecated
//#define GL_FRAGMENT_DEPTH                                           0x8452 // deprecated
//#define GL_CURRENT_FOG_COORDINATE                                   0x8453 // deprecated
//#define GL_FOG_COORDINATE_ARRAY_TYPE                                0x8454 // deprecated
//#define GL_FOG_COORDINATE_ARRAY_STRIDE                              0x8455 // deprecated
//#define GL_FOG_COORDINATE_ARRAY_POINTER                             0x8456 // deprecated
//#define GL_FOG_COORDINATE_ARRAY                                     0x8457 // deprecated
//#define GL_COLOR_SUM                                                0x8458 // deprecated
//#define GL_CURRENT_SECONDARY_COLOR                                  0x8459 // deprecated
//#define GL_SECONDARY_COLOR_ARRAY_SIZE                               0x845A // deprecated
//#define GL_SECONDARY_COLOR_ARRAY_TYPE                               0x845B // deprecated
//#define GL_SECONDARY_COLOR_ARRAY_STRIDE                             0x845C // deprecated
//#define GL_SECONDARY_COLOR_ARRAY_POINTER                            0x845D // deprecated
//#define GL_SECONDARY_COLOR_ARRAY                                    0x845E // deprecated
#define GL_MAX_TEXTURE_LOD_BIAS                                     0x84FD /**< @brief OpenGL 1.4 enum */
//#define GL_TEXTURE_FILTER_CONTROL                                   0x8500 // deprecated
//#define GL_TEXTURE_LOD_BIAS                                         0x8501 // deprecated
#define GL_INCR_WRAP                                                0x8507 /**< @brief OpenGL 1.4 enum */
#define GL_DECR_WRAP                                                0x8508 /**< @brief OpenGL 1.4 enum */
#define GL_TEXTURE_DEPTH_SIZE                                       0x884A /**< @brief OpenGL 1.4 enum */
//#define GL_DEPTH_TEXTURE_MODE                                       0x884B // deprecated
#define GL_TEXTURE_COMPARE_MODE                                     0x884C /**< @brief OpenGL 1.4 enum */
#define GL_TEXTURE_COMPARE_FUNC                                     0x884D /**< @brief OpenGL 1.4 enum */
//#define GL_COMPARE_R_TO_TEXTURE                                     0x884E // deprecated (in OpenGL 3.0, use GL_COMPARE_REF_TO_TEXTURE)

/**
 *  @brief specify pixel arithmetic for RGB and alpha components separately
 *  @param[in] srcRGB Specifies how the red, green, and blue blending factors are computed. The
 *      initial value is GL_ONE.
 *  @param[in] dstRGB Specifies how the red, green, and blue destination blending factors are
 *      computed. The initial value is GL_ZERO.
 *  @param[in] srcAlpha Specified how the alpha source blending factor is computed. The initial
 *      value is GL_ONE.
 *  @param[in] dstAlpha Specified how the alpha destination blending factor is computed. The initial
 *      value is GL_ZERO.
 */
extern void (GLApi *GLFuncPtrName(glBlendFuncSeparate))(GLenum srcRGB, GLenum dstRGB, GLenum srcAlpha, GLenum dstAlpha);

// functions below are deprecated
/*extern void (GLApi *GLFuncPtrName(glFogCoordf))(GLfloat);
extern void (GLApi *GLFuncPtrName(glFogCoordfv))(const GLfloat*);
extern void (GLApi *GLFuncPtrName(glFogCoordd))(GLdouble);
extern void (GLApi *GLFuncPtrName(glFogCoorddv))(const GLdouble*);
extern void (GLApi *GLFuncPtrName(glFogCoordPointer))(GLenum, GLsizei, const GLvoid*);*/

/**
 *  @brief render multiple sets of primitives from array data
 *  @param[in] mode Specifies what kind of primitives to render. Symbolic constants GL_POINTS,
 *      GL_LINE_STRIP, GL_LINE_LOOP, GL_LINES, GL_LINE_STRIP_ADJACENCY, GL_LINES_ADJACENCY,
 *      GL_TRIANGLE_STRIP, GL_TRIANGLE_FAN, GL_TRIANGLES, GL_TRIANGLE_STRIP_ADJACENCY,
 *      GL_TRIANGLES_ADJACENCY and GL_PATCHES are accepted.
 *  @param[out] first Points to an array of starting indices in the enabled arrays.
 *  @param[out] count Points to an array of the number of indices to be rendered.
 *  @param[in] primcount Specifies the size of the first and count
 */
extern void (GLApi *GLFuncPtrName(glMultiDrawArrays))(GLenum mode, int *first, GLsizei *count, GLsizei primcount);

/**
 *  @brief render multiple sets of primitives by specifying indices of array data elements
 *  @param[in] mode Specifies what kind of primitives to render. Symbolic constants GL_POINTS,
 *      GL_LINE_STRIP, GL_LINE_LOOP, GL_LINES, GL_LINE_STRIP_ADJACENCY, GL_LINES_ADJACENCY,
 *      GL_TRIANGLE_STRIP, GL_TRIANGLE_FAN, GL_TRIANGLES, GL_TRIANGLE_STRIP_ADJACENCY,
 *      GL_TRIANGLES_ADJACENCY and GL_PATCHES are accepted.
 *  @param[in] count Points to an array of the elements counts.
 *  @param[in] type Specifies the type of the values in indices. Must be one of GL_UNSIGNED_BYTE,
 *      GL_UNSIGNED_SHORT, or GL_UNSIGNED_INT.
 *  @param[in] indices Specifies a pointer to the location where the indices are stored.
 *  @param[in] primcount Specifies the size of the count array.
 */
extern void (GLApi *GLFuncPtrName(glMultiDrawElements))(GLenum mode, const GLsizei *count, GLenum type, const void **indices, GLsizei primcount);

/**
 *  @brief specify point parameters
 *  @param[in] pname Specifies a single-valued point parameter. GL_POINT_FADE_THRESHOLD_SIZE, and
 *      GL_POINT_SPRITE_COORD_ORIGIN are accepted.
 *  @param[in] param Specifies the value that pname will be set to.
 */
extern void (GLApi *GLFuncPtrName(glPointParameterf))(GLenum pname, float param);

/**
 *  @brief specify point parameters
 *  @param[in] pname Specifies a point parameter. GL_POINT_FADE_THRESHOLD_SIZE, and
 *      GL_POINT_SPRITE_COORD_ORIGIN are accepted.
 *  @param[in] params Specifies the value to be assigned to pname..
 */
extern void (GLApi *GLFuncPtrName(glPointParameterfv))(GLenum pname, const float *params);

/**
 *  @brief specify point parameters
 *  @param[in] pname Specifies a single-valued point parameter. GL_POINT_FADE_THRESHOLD_SIZE, and
 *      GL_POINT_SPRITE_COORD_ORIGIN are accepted.
 *  @param[in] param Specifies the value that pname will be set to.
 */
extern void (GLApi *GLFuncPtrName(glPointParameteri))(GLenum pname, int param);

/**
 *  @brief specify point parameters
 *  @param[in] pname Specifies a point parameter. GL_POINT_FADE_THRESHOLD_SIZE, and
 *      GL_POINT_SPRITE_COORD_ORIGIN are accepted.
 *  @param[in] params Specifies the value to be assigned to pname..
 */
extern void (GLApi *GLFuncPtrName(glPointParameteriv))(GLenum pname, const int *params);

// functions below are deprecated
/*extern void (GLApi *GLFuncPtrName(glSecondaryColor3b))(GLbyte, GLbyte, GLbyte);
extern void (GLApi *GLFuncPtrName(glSecondaryColor3bv))(const GLbyte*);
extern void (GLApi *GLFuncPtrName(glSecondaryColor3d))(GLdouble, GLdouble, GLdouble);
extern void (GLApi *GLFuncPtrName(glSecondaryColor3dv))(const GLdouble*);
extern void (GLApi *GLFuncPtrName(glSecondaryColor3f))(GLfloat, GLfloat, GLfloat);
extern void (GLApi *GLFuncPtrName(glSecondaryColor3fv))(const GLfloat*);
extern void (GLApi *GLFuncPtrName(glSecondaryColor3i))(GLint, GLint, GLint);
extern void (GLApi *GLFuncPtrName(glSecondaryColor3iv))(const GLint*);
extern void (GLApi *GLFuncPtrName(glSecondaryColor3s))(GLshort, GLshort, GLshort);
extern void (GLApi *GLFuncPtrName(glSecondaryColor3sv))(const GLshort*);
extern void (GLApi *GLFuncPtrName(glSecondaryColor3ub))(GLubyte, GLubyte, GLubyte);
extern void (GLApi *GLFuncPtrName(glSecondaryColor3ubv))(const GLubyte*);
extern void (GLApi *GLFuncPtrName(glSecondaryColor3ui))(GLuint, GLuint, GLuint);
extern void (GLApi *GLFuncPtrName(glSecondaryColor3uiv))(const GLuint*);
extern void (GLApi *GLFuncPtrName(glSecondaryColor3us))(GLushort, GLushort, GLushort);
extern void (GLApi *GLFuncPtrName(glSecondaryColor3usv))(const GLushort*);
extern void (GLApi *GLFuncPtrName(glSecondaryColorPointer))(GLint, GLenum, GLsizei, const GLvoid*);
extern void (GLApi *GLFuncPtrName(glWindowPos2d))(GLdouble, GLdouble);
extern void (GLApi *GLFuncPtrName(glWindowPos2dv))(const GLdouble*);
extern void (GLApi *GLFuncPtrName(glWindowPos2f))(GLfloat, GLfloat);
extern void (GLApi *GLFuncPtrName(glWindowPos2fv))(const GLfloat*);
extern void (GLApi *GLFuncPtrName(glWindowPos2i))(GLint, GLint);
extern void (GLApi *GLFuncPtrName(glWindowPos2iv))(const GLint*);
extern void (GLApi *GLFuncPtrName(glWindowPos2s))(GLshort, GLshort);
extern void (GLApi *GLFuncPtrName(glWindowPos2sv))(const GLshort*);
extern void (GLApi *GLFuncPtrName(glWindowPos3d))(GLdouble, GLdouble, GLdouble);
extern void (GLApi *GLFuncPtrName(glWindowPos3dv))(const GLdouble*);
extern void (GLApi *GLFuncPtrName(glWindowPos3f))(GLfloat, GLfloat, GLfloat);
extern void (GLApi *GLFuncPtrName(glWindowPos3fv))(const GLfloat*);
extern void (GLApi *GLFuncPtrName(glWindowPos3i))(GLint, GLint, GLint);
extern void (GLApi *GLFuncPtrName(glWindowPos3iv))(const GLint*);
extern void (GLApi *GLFuncPtrName(glWindowPos3s))(GLshort, GLshort, GLshort);
extern void (GLApi *GLFuncPtrName(glWindowPos3sv))(const GLshort*);*/

/**
 *  @def glBlendFuncSeparate
 *  @brief specify pixel arithmetic for RGB and alpha components separately
 *  @param[in] srcRGB Specifies how the red, green, and blue blending factors are computed. The
 *      initial value is GL_ONE.
 *  @param[in] dstRGB Specifies how the red, green, and blue destination blending factors are
 *      computed. The initial value is GL_ZERO.
 *  @param[in] srcAlpha Specified how the alpha source blending factor is computed. The initial
 *      value is GL_ONE.
 *  @param[in] dstAlpha Specified how the alpha destination blending factor is computed. The initial
 *      value is GL_ZERO.
 */
#define glBlendFuncSeparate(srcRGB,dstRGB,srcAlpha,dstAlpha) do { GLFuncPtrName(glBlendFuncSeparate)(srcRGB, dstRGB, srcAlpha, dstAlpha); GL_AUTOCHECK_BLOCK("glBlendFuncSeparate", __FILE__, __LINE__); } while(0)

/**
 *  @def glMultiDrawArrays
 *  @brief render multiple sets of primitives from array data
 *  @param[in] mode Specifies what kind of primitives to render. Symbolic constants GL_POINTS,
 *      GL_LINE_STRIP, GL_LINE_LOOP, GL_LINES, GL_LINE_STRIP_ADJACENCY, GL_LINES_ADJACENCY,
 *      GL_TRIANGLE_STRIP, GL_TRIANGLE_FAN, GL_TRIANGLES, GL_TRIANGLE_STRIP_ADJACENCY,
 *      GL_TRIANGLES_ADJACENCY and GL_PATCHES are accepted.
 *  @param[out] first Points to an array of starting indices in the enabled arrays.
 *  @param[out] count Points to an array of the number of indices to be rendered.
 *  @param[in] primcount Specifies the size of the first and count
 */
#define glMultiDrawArrays(mode,first,count,primcount) do { GLFuncPtrName(glMultiDrawArrays)(mode, first, count, primcount); GL_AUTOCHECK_BLOCK("glMultiDrawArrays", __FILE__, __LINE__); } while(0)

/**
 *  @def glMultiDrawElements
 *  @brief render multiple sets of primitives by specifying indices of array data elements
 *  @param[in] mode Specifies what kind of primitives to render. Symbolic constants GL_POINTS,
 *      GL_LINE_STRIP, GL_LINE_LOOP, GL_LINES, GL_LINE_STRIP_ADJACENCY, GL_LINES_ADJACENCY,
 *      GL_TRIANGLE_STRIP, GL_TRIANGLE_FAN, GL_TRIANGLES, GL_TRIANGLE_STRIP_ADJACENCY,
 *      GL_TRIANGLES_ADJACENCY and GL_PATCHES are accepted.
 *  @param[in] count Points to an array of the elements counts.
 *  @param[in] type Specifies the type of the values in indices. Must be one of GL_UNSIGNED_BYTE,
 *      GL_UNSIGNED_SHORT, or GL_UNSIGNED_INT.
 *  @param[in] indices Specifies a pointer to the location where the indices are stored.
 *  @param[in] primcount Specifies the size of the count array.
 */
#define glMultiDrawElements(mode,count,type,indices,primcount) do { GLFuncPtrName(glMultiDrawElements)(mode, count, type, indices, primcount); GL_AUTOCHECK_BLOCK("glMultiDrawElements", __FILE__, __LINE__); } while(0)

/**
 *  @def glPointParameterf
 *  @brief specify point parameters
 *  @param[in] pname Specifies a single-valued point parameter. GL_POINT_FADE_THRESHOLD_SIZE, and
 *      GL_POINT_SPRITE_COORD_ORIGIN are accepted.
 *  @param[in] param Specifies the value that pname will be set to.
 */
#define glPointParameterf(pname,param) do { GLFuncPtrName(glPointParameterf)(pname, param); GL_AUTOCHECK_BLOCK("glPointParameterf", __FILE__, __LINE__); } while(0)

/**
 *  @def glPointParameterfv
 *  @brief specify point parameters
 *  @param[in] pname Specifies a point parameter. GL_POINT_FADE_THRESHOLD_SIZE, and
 *      GL_POINT_SPRITE_COORD_ORIGIN are accepted.
 *  @param[in] params Specifies the value to be assigned to pname..
 */
#define glPointParameterfv(pname,params) do { GLFuncPtrName(glPointParameterfv)(pname, params); GL_AUTOCHECK_BLOCK("glPointParameterfv", __FILE__, __LINE__); } while(0)

/**
 *  @def glPointParameteri
 *  @brief specify point parameters
 *  @param[in] pname Specifies a single-valued point parameter. GL_POINT_FADE_THRESHOLD_SIZE, and
 *      GL_POINT_SPRITE_COORD_ORIGIN are accepted.
 *  @param[in] param Specifies the value that pname will be set to.
 */
#define glPointParameteri(pname,param) do { GLFuncPtrName(glPointParameteri)(pname, param); GL_AUTOCHECK_BLOCK("glPointParameteri", __FILE__, __LINE__); } while(0)

/**
 *  @def glPointParameteriv
 *  @brief specify point parameters
 *  @param[in] pname Specifies a point parameter. GL_POINT_FADE_THRESHOLD_SIZE, and
 *      GL_POINT_SPRITE_COORD_ORIGIN are accepted.
 *  @param[in] params Specifies the value to be assigned to pname..
 */
#define glPointParameteriv(pname,params) do { GLFuncPtrName(glPointParameteriv)(pname, params); GL_AUTOCHECK_BLOCK("glPointParameteriv", __FILE__, __LINE__); } while(0)

#endif // GL_VERSION_1_4

//                              --- ~OpenGL 1.4 ---

