//                              --- OpenGL 2.1 ---

/**
 *  @def GL_VERSION_2_1
 *  @brief if defined, OpenGL 2.1 entry points are available
 *  @note This does not necessarily mean it is also supported.
 */
#ifndef GL_VERSION_2_1
#define GL_VERSION_2_1 1

/**
 *  @def __GENERATE_GL_21__
 *  @brief if defined, code to initialize OpenGL 2.1 entry points is compiled
 */
#define __GENERATE_GL_21__

extern bool GLEH_OPENGL_2_1; /**< @brief set if OpenGL 2.1 is supported */

//#define GL_CURRENT_RASTER_SECONDARY_COLOR                           0x845F // deprecated
#define GL_PIXEL_PACK_BUFFER                                        0x88EB /**< @brief OpenGL 2.1 enum */
#define GL_PIXEL_UNPACK_BUFFER                                      0x88EC /**< @brief OpenGL 2.1 enum */
#define GL_PIXEL_PACK_BUFFER_BINDING                                0x88ED /**< @brief OpenGL 2.1 enum */
#define GL_PIXEL_UNPACK_BUFFER_BINDING                              0x88EF /**< @brief OpenGL 2.1 enum */
#define GL_FLOAT_MAT2x3                                             0x8B65 /**< @brief OpenGL 2.1 enum */
#define GL_FLOAT_MAT2x4                                             0x8B66 /**< @brief OpenGL 2.1 enum */
#define GL_FLOAT_MAT3x2                                             0x8B67 /**< @brief OpenGL 2.1 enum */
#define GL_FLOAT_MAT3x4                                             0x8B68 /**< @brief OpenGL 2.1 enum */
#define GL_FLOAT_MAT4x2                                             0x8B69 /**< @brief OpenGL 2.1 enum */
#define GL_FLOAT_MAT4x3                                             0x8B6A /**< @brief OpenGL 2.1 enum */
#define GL_SRGB                                                     0x8C40 /**< @brief OpenGL 2.1 enum */
#define GL_SRGB8                                                    0x8C41 /**< @brief OpenGL 2.1 enum */
#define GL_SRGB_ALPHA                                               0x8C42 /**< @brief OpenGL 2.1 enum */
#define GL_SRGB8_ALPHA8                                             0x8C43 /**< @brief OpenGL 2.1 enum */
//#define GL_SLUMINANCE_ALPHA                                         0x8C44 // deprecated
//#define GL_SLUMINANCE8_ALPHA8                                       0x8C45 // deprecated
//#define GL_SLUMINANCE                                               0x8C46 // deprecated
//#define GL_SLUMINANCE8                                              0x8C47 // deprecated
#define GL_COMPRESSED_SRGB                                          0x8C48 /**< @brief OpenGL 2.1 enum */
#define GL_COMPRESSED_SRGB_ALPHA                                    0x8C49 /**< @brief OpenGL 2.1 enum */
//#define GL_COMPRESSED_SLUMINANCE                                    0x8C4A // deprecated
//#define GL_COMPRESSED_SLUMINANCE_ALPHA                              0x8C4B // deprecated

/**
 *  @brief Specify the value of a uniform variable for the current program object
 *  @param[in] location Specifies the location of the uniform value to be modified.
 *  @param[in] count Specifies the number of matrices that are to be modified. This should be 1 if
 *      the targeted uniform variable is not an array of matrices, and 1 or more if it is an array
 *      of matrices.
 *  @param[in] transpose Specifies whether to transpose the matrix as the values are loaded into the
 *      uniform variable.
 *  @param[in] value Specifies a pointer to an array of count values that will be used to update the
 *      specified uniform variable.
 */
extern void (GLApi *GLFuncPtrName(glUniformMatrix2x3fv))(int location, GLsizei count, GLboolean transpose, const float *value);

/**
 *  @brief Specify the value of a uniform variable for the current program object
 *  @param[in] location Specifies the location of the uniform value to be modified.
 *  @param[in] count Specifies the number of matrices that are to be modified. This should be 1 if
 *      the targeted uniform variable is not an array of matrices, and 1 or more if it is an array
 *      of matrices.
 *  @param[in] transpose Specifies whether to transpose the matrix as the values are loaded into the
 *      uniform variable.
 *  @param[in] value Specifies a pointer to an array of count values that will be used to update the
 *      specified uniform variable.
 */
extern void (GLApi *GLFuncPtrName(glUniformMatrix3x2fv))(int location, GLsizei count, GLboolean transpose, const float *value);

/**
 *  @brief Specify the value of a uniform variable for the current program object
 *  @param[in] location Specifies the location of the uniform value to be modified.
 *  @param[in] count Specifies the number of matrices that are to be modified. This should be 1 if
 *      the targeted uniform variable is not an array of matrices, and 1 or more if it is an array
 *      of matrices.
 *  @param[in] transpose Specifies whether to transpose the matrix as the values are loaded into the
 *      uniform variable.
 *  @param[in] value Specifies a pointer to an array of count values that will be used to update the
 *      specified uniform variable.
 */
extern void (GLApi *GLFuncPtrName(glUniformMatrix2x4fv))(int location, GLsizei count, GLboolean transpose, const float *value);

/**
 *  @brief Specify the value of a uniform variable for the current program object
 *  @param[in] location Specifies the location of the uniform value to be modified.
 *  @param[in] count Specifies the number of matrices that are to be modified. This should be 1 if
 *      the targeted uniform variable is not an array of matrices, and 1 or more if it is an array
 *      of matrices.
 *  @param[in] transpose Specifies whether to transpose the matrix as the values are loaded into the
 *      uniform variable.
 *  @param[in] value Specifies a pointer to an array of count values that will be used to update the
 *      specified uniform variable.
 */
extern void (GLApi *GLFuncPtrName(glUniformMatrix4x2fv))(int location, GLsizei count, GLboolean transpose, const float *value);

/**
 *  @brief Specify the value of a uniform variable for the current program object
 *  @param[in] location Specifies the location of the uniform value to be modified.
 *  @param[in] count Specifies the number of matrices that are to be modified. This should be 1 if
 *      the targeted uniform variable is not an array of matrices, and 1 or more if it is an array
 *      of matrices.
 *  @param[in] transpose Specifies whether to transpose the matrix as the values are loaded into the
 *      uniform variable.
 *  @param[in] value Specifies a pointer to an array of count values that will be used to update the
 *      specified uniform variable.
 */
extern void (GLApi *GLFuncPtrName(glUniformMatrix3x4fv))(int location, GLsizei count, GLboolean transpose, const float *value);

/**
 *  @brief Specify the value of a uniform variable for the current program object
 *  @param[in] location Specifies the location of the uniform value to be modified.
 *  @param[in] count Specifies the number of matrices that are to be modified. This should be 1 if
 *      the targeted uniform variable is not an array of matrices, and 1 or more if it is an array
 *      of matrices.
 *  @param[in] transpose Specifies whether to transpose the matrix as the values are loaded into the
 *      uniform variable.
 *  @param[in] value Specifies a pointer to an array of count values that will be used to update the
 *      specified uniform variable.
 */
extern void (GLApi *GLFuncPtrName(glUniformMatrix4x3fv))(int location, GLsizei count, GLboolean transpose, const float *value);

/**
 *  @def glUniformMatrix2x3fv
 *  @brief Specify the value of a uniform variable for the current program object
 *  @param[in] location Specifies the location of the uniform value to be modified.
 *  @param[in] count Specifies the number of matrices that are to be modified. This should be 1 if
 *      the targeted uniform variable is not an array of matrices, and 1 or more if it is an array
 *      of matrices.
 *  @param[in] transpose Specifies whether to transpose the matrix as the values are loaded into the
 *      uniform variable.
 *  @param[in] value Specifies a pointer to an array of count values that will be used to update the
 *      specified uniform variable.
 */
#define glUniformMatrix2x3fv(location,count,transpose,value) do { GLFuncPtrName(glUniformMatrix2x3fv)(location, count, transpose, value); GL_AUTOCHECK_BLOCK("glUniformMatrix2x3fv", __FILE__, __LINE__); } while(0)

/**
 *  @def glUniformMatrix3x2fv
 *  @brief Specify the value of a uniform variable for the current program object
 *  @param[in] location Specifies the location of the uniform value to be modified.
 *  @param[in] count Specifies the number of matrices that are to be modified. This should be 1 if
 *      the targeted uniform variable is not an array of matrices, and 1 or more if it is an array
 *      of matrices.
 *  @param[in] transpose Specifies whether to transpose the matrix as the values are loaded into the
 *      uniform variable.
 *  @param[in] value Specifies a pointer to an array of count values that will be used to update the
 *      specified uniform variable.
 */
#define glUniformMatrix3x2fv(location,count,transpose,value) do { GLFuncPtrName(glUniformMatrix3x2fv)(location, count, transpose, value); GL_AUTOCHECK_BLOCK("glUniformMatrix3x2fv", __FILE__, __LINE__); } while(0)

/**
 *  @def glUniformMatrix2x4fv
 *  @brief Specify the value of a uniform variable for the current program object
 *  @param[in] location Specifies the location of the uniform value to be modified.
 *  @param[in] count Specifies the number of matrices that are to be modified. This should be 1 if
 *      the targeted uniform variable is not an array of matrices, and 1 or more if it is an array
 *      of matrices.
 *  @param[in] transpose Specifies whether to transpose the matrix as the values are loaded into the
 *      uniform variable.
 *  @param[in] value Specifies a pointer to an array of count values that will be used to update the
 *      specified uniform variable.
 */
#define glUniformMatrix2x4fv(location,count,transpose,value) do { GLFuncPtrName(glUniformMatrix2x4fv)(location, count, transpose, value); GL_AUTOCHECK_BLOCK("glUniformMatrix2x4fv", __FILE__, __LINE__); } while(0)

/**
 *  @def glUniformMatrix4x2fv
 *  @brief Specify the value of a uniform variable for the current program object
 *  @param[in] location Specifies the location of the uniform value to be modified.
 *  @param[in] count Specifies the number of matrices that are to be modified. This should be 1 if
 *      the targeted uniform variable is not an array of matrices, and 1 or more if it is an array
 *      of matrices.
 *  @param[in] transpose Specifies whether to transpose the matrix as the values are loaded into the
 *      uniform variable.
 *  @param[in] value Specifies a pointer to an array of count values that will be used to update the
 *      specified uniform variable.
 */
#define glUniformMatrix4x2fv(location,count,transpose,value) do { GLFuncPtrName(glUniformMatrix4x2fv)(location, count, transpose, value); GL_AUTOCHECK_BLOCK("glUniformMatrix4x2fv", __FILE__, __LINE__); } while(0)

/**
 *  @def glUniformMatrix3x4fv
 *  @brief Specify the value of a uniform variable for the current program object
 *  @param[in] location Specifies the location of the uniform value to be modified.
 *  @param[in] count Specifies the number of matrices that are to be modified. This should be 1 if
 *      the targeted uniform variable is not an array of matrices, and 1 or more if it is an array
 *      of matrices.
 *  @param[in] transpose Specifies whether to transpose the matrix as the values are loaded into the
 *      uniform variable.
 *  @param[in] value Specifies a pointer to an array of count values that will be used to update the
 *      specified uniform variable.
 */
#define glUniformMatrix3x4fv(location,count,transpose,value) do { GLFuncPtrName(glUniformMatrix3x4fv)(location, count, transpose, value); GL_AUTOCHECK_BLOCK("glUniformMatrix3x4fv", __FILE__, __LINE__); } while(0)

/**
 *  @def glUniformMatrix4x3fv
 *  @brief Specify the value of a uniform variable for the current program object
 *  @param[in] location Specifies the location of the uniform value to be modified.
 *  @param[in] count Specifies the number of matrices that are to be modified. This should be 1 if
 *      the targeted uniform variable is not an array of matrices, and 1 or more if it is an array
 *      of matrices.
 *  @param[in] transpose Specifies whether to transpose the matrix as the values are loaded into the
 *      uniform variable.
 *  @param[in] value Specifies a pointer to an array of count values that will be used to update the
 *      specified uniform variable.
 */
#define glUniformMatrix4x3fv(location,count,transpose,value) do { GLFuncPtrName(glUniformMatrix4x3fv)(location, count, transpose, value); GL_AUTOCHECK_BLOCK("glUniformMatrix4x3fv", __FILE__, __LINE__); } while(0)

#endif // GL_VERSION_2_1

//                              --- ~OpenGL 2.1 ---

