
//                              --- GL_ARB_vertex_array_object ---

/**
 *  @def GL_ARB_vertex_array_object
 *  @brief if defined, GL_ARB_vertex_array_object entry points are available
 */
#ifndef GL_ARB_vertex_array_object
#define GL_ARB_vertex_array_object 1

/**
 *  @def __GENERATE_GL_ARB_vertex_array_object
 *  @brief if defined, GL_ARB_vertex_array_object entry points are available
 */
#define __GENERATE_GL_ARB_vertex_array_object

extern bool GLEH_ARB_vertex_array_object; /**< @brief set if GL_ARB_vertex_array_object is supported */

#define GL_VERTEX_ARRAY_BINDING                                     0x85b5 /**< @brief GL_ARB_vertex_array_object enum **/

/**
 *  @brief bind a vertex array object
 *  @param[in] array Specifies the name of the vertex array to bind.
 */
extern void (GLApi *GLFuncPtrName(glBindVertexArray))(GLuint array);

/**
 *  @brief delete vertex array objects
 *  @param[in] n Specifies the number of vertex array objects to be deleted.
 *  @param[in] arrays Specifies the address of an array containing the n names of the objects to be deleted.
 */
extern void (GLApi *GLFuncPtrName(glDeleteVertexArrays))(GLsizei n, const GLuint *arrays);

/**
 *  @brief generate vertex array object names
 *  @param[in] n Specifies the number of vertex array object names to generate.
 *  @param[out] arrays Specifies an array in which the generated vertex array object names are stored.
 */
extern void (GLApi *GLFuncPtrName(glGenVertexArrays))(GLsizei n, GLuint *arrays);

/**
 *  @brief determine if a name corresponds to a vertex array object
 *  @param[in] array Specifies a value that may be the name of a vertex array object.
 */
extern GLboolean (GLApi *GLFuncPtrName(glIsVertexArray))(GLuint array);

inline GLboolean glIsVertexArray_glachwrap(GLuint array, const char *p_s_filename, int n_line) { GLboolean result = GLFuncPtrName(glIsVertexArray)(array); GL_AUTOCHECK_BLOCK("glIsVertexArray", p_s_filename, n_line); return result; }

/**
 *  @def glBindVertexArray
 *  @brief bind a vertex array object
 *  @param[in] array Specifies the name of the vertex array to bind.
 */
#define glBindVertexArray(array) do { GLFuncPtrName(glBindVertexArray)(array); GL_AUTOCHECK_BLOCK("glBindVertexArray", __FILE__, __LINE__); } while(0)

/**
 *  @def glDeleteVertexArrays
 *  @brief delete vertex array objects
 *  @param[in] n Specifies the number of vertex array objects to be deleted.
 *  @param[in] arrays Specifies the address of an array containing the n names of the objects to be deleted.
 */
#define glDeleteVertexArrays(n,arrays) do { GLFuncPtrName(glDeleteVertexArrays)(n, arrays); GL_AUTOCHECK_BLOCK("glDeleteVertexArrays", __FILE__, __LINE__); } while(0)

/**
 *  @def glGenVertexArrays
 *  @brief generate vertex array object names
 *  @param[in] n Specifies the number of vertex array object names to generate.
 *  @param[out] arrays Specifies an array in which the generated vertex array object names are stored.
 */
#define glGenVertexArrays(n,arrays) do { GLFuncPtrName(glGenVertexArrays)(n, arrays); GL_AUTOCHECK_BLOCK("glGenVertexArrays", __FILE__, __LINE__); } while(0)

/**
 *  @def glIsVertexArray
 *  @brief determine if a name corresponds to a vertex array object
 *  @param[in] array Specifies a value that may be the name of a vertex array object.
 */
#define glIsVertexArray(array) GLAutoCheckFuncPtrName(glIsVertexArray)(array GLAutoCheckExtraParams)

#if !defined(GLES20EMU_CPP)
// do not patch this for GLES20Emu.cpp

extern GLuint __n_gles20emupatch_default_vertex_array;
// default vertex array in place of array 0 (array 0 is not allowed to be used for actual rendering in new OpenGL specifications)

inline void __gles20emupatch_glGetIntegerv(GLenum pname, GLint* params) { glGetIntegerv(pname, params); if(pname == GL_VERTEX_ARRAY_BINDING && params != 0 && *params == __n_gles20emupatch_default_vertex_array) *params = 0; /* obfuscate the default vertex array and return 0 */ }
inline void __gles20emupatch_glBindVertexArray(GLuint array) { if(array) glBindVertexArray(array); else glBindVertexArray(__n_gles20emupatch_default_vertex_array); }
// patched function prototypes

#ifdef __HAVE_GL_AUTOCHECK
#undef glGetIntegerv
#endif // __HAVE_GL_AUTOCHECK
// statically linked functions are only redefined for autocheck

#undef glBindVertexArray
// dynamically linked functions are always redefined

#define glGetIntegerv(pname,params) __gles20emupatch_glGetIntegerv((pname), (params))
#define glBindVertexArray(array) __gles20emupatch_glBindVertexArray((array))
// use patched functions

#endif // !GLES20EMU_CPP

#endif // GL_ARB_vertex_array_object

//                              --- ~GL_ARB_vertex_array_object ---

